@@ -1,4 +1,4 @@
-/*  Copyright (C) 2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2024 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -37,14 +37,30 @@ enum knot_error {
 	KNOT_ENOTSUP       = -ENOTSUP,
 	KNOT_EBUSY         = -EBUSY,
 	KNOT_EAGAIN        = -EAGAIN,
+	KNOT_ENOBUFS       = -ENOBUFS,
+	KNOT_EMFILE        = -EMFILE,
+	KNOT_ENFILE        = -ENFILE,
 	KNOT_EACCES        = -EACCES,
-	KNOT_ECONNREFUSED  = -ECONNREFUSED,
 	KNOT_EISCONN       = -EISCONN,
+	KNOT_ECONNREFUSED  = -ECONNREFUSED,
+	KNOT_EALREADY      = -EALREADY,
+	KNOT_ECONNRESET    = -ECONNRESET,
+	KNOT_ECONNABORTED  = -ECONNABORTED,
+	KNOT_ENETRESET     = -ENETRESET,
+	KNOT_EHOSTUNREACH  = -EHOSTUNREACH,
+	KNOT_ENETUNREACH   = -ENETUNREACH,
+	KNOT_EHOSTDOWN     = -EHOSTDOWN,
+	KNOT_ENETDOWN      = -ENETDOWN,
 	KNOT_EADDRINUSE    = -EADDRINUSE,
 	KNOT_ENOENT        = -ENOENT,
 	KNOT_EEXIST        = -EEXIST,
 	KNOT_ERANGE        = -ERANGE,
 	KNOT_EADDRNOTAVAIL = -EADDRNOTAVAIL,
+	KNOT_ENOTDIR       = -ENOTDIR,
+	KNOT_EPERM         = -EPERM,
+	KNOT_EROFS         = -EROFS,
+
+	KNOT_ERRNO_ERROR   = -500,
 
 	KNOT_ERROR_MIN = -1000,
 
@@ -67,6 +83,7 @@ enum knot_error {
 	KNOT_ENOMASTER,
 	KNOT_EPREREQ,
 	KNOT_ETTL,
+	KNOT_ENOTTL,
 	KNOT_ENOXFR,
 	KNOT_EDENIED,
 	KNOT_ECONN,
@@ -81,9 +98,24 @@ enum knot_error {
 	KNOT_ESOAINVAL,
 	KNOT_ETRAIL,
 	KNOT_EPROCESSING,
+	KNOT_EPROGRESS,
+	KNOT_ELOOP,
+	KNOT_EPROGRAM,
+	KNOT_EFD,
+	KNOT_ENOPARAM,
+	KNOT_EXPARAM,
+	KNOT_EEMPTYZONE,
+	KNOT_ENODB,
+	KNOT_EUNREACH,
+	KNOT_EBADCERTKEY,
+	KNOT_EFACCES,
+	KNOT_EBACKUPDATA,
+
+	KNOT_GENERAL_ERROR = -900,
 
 	/* Control states. */
 	KNOT_CTL_ESTOP,
+	KNOT_CTL_EZONE,
 
 	/* Network errors. */
 	KNOT_NET_EADDR,
@@ -108,6 +140,11 @@ enum knot_error {
 	/* DNSSEC errors. */
 	KNOT_DNSSEC_EMISSINGKEYTYPE,
 	KNOT_DNSSEC_ENOKEY,
+	KNOT_DNSSEC_ENOSIG,
+	KNOT_DNSSEC_ENONSEC,
+	KNOT_DNSSEC_ENSEC_BITMAP,
+	KNOT_DNSSEC_ENSEC_CHAIN,
+	KNOT_DNSSEC_ENSEC3_OPTOUT,
 
 	/* Yparser errors. */
 	KNOT_YP_ECHAR_TAB,
@@ -138,9 +175,9 @@ enum knot_error {
 	KNOT_INVALID_KEY_NAME,
 	KNOT_NO_PUBLIC_KEY,
 	KNOT_NO_PRIVATE_KEY,
-
-	/* Another general errors. */
-	KNOT_ELOOP,
+	KNOT_NO_READY_KEY,
+	KNOT_DNSSEC_EKEYTAG_LIMIT,
+	KNOT_DNSSEC_EXTRA_NSEC,
 
 	KNOT_ERROR_MAX = -501
 };
@@ -148,11 +185,12 @@ enum knot_error {
 /*!
  * \brief Map POSIX errno code to Knot error code.
  *
- * \param code Errno code to transform (set -1 to use the current errno).
+ * \param code       Errno code to transform (set -1 to use the current errno).
+ * \param dflt_error Default return value, if code is unknown.
  *
- * \return Mapped errno or KNOT_ERROR if unknown.
+ * \return Mapped errno or the value of dflt_error if unknown.
  */
-inline static int knot_map_errno_code(int code)
+inline static int knot_map_errno_code_def(int code, int dflt_error)
 {
 	if (code < 0) {
 		code = errno;
@@ -170,17 +208,28 @@ inline static int knot_map_errno_code(int code)
 		ERR_ITEM(ENOTSUP),
 		ERR_ITEM(EBUSY),
 		ERR_ITEM(EAGAIN),
+		ERR_ITEM(ENOBUFS),
+		ERR_ITEM(EMFILE),
+		ERR_ITEM(ENFILE),
 		ERR_ITEM(EACCES),
-		ERR_ITEM(ECONNREFUSED),
 		ERR_ITEM(EISCONN),
+		ERR_ITEM(ECONNREFUSED),
+		ERR_ITEM(EALREADY),
+		ERR_ITEM(ECONNRESET),
+		ERR_ITEM(ECONNABORTED),
+		ERR_ITEM(ENETRESET),
+		ERR_ITEM(EHOSTUNREACH),
+		ERR_ITEM(ENETUNREACH),
+		ERR_ITEM(EHOSTDOWN),
+		ERR_ITEM(ENETDOWN),
 		ERR_ITEM(EADDRINUSE),
 		ERR_ITEM(ENOENT),
 		ERR_ITEM(EEXIST),
 		ERR_ITEM(ERANGE),
 		ERR_ITEM(EADDRNOTAVAIL),
 
-		/* Terminator - default value. */
-		{ 0, KNOT_ERROR }
+		/* Terminator - the value isn't used. */
+		{ 0, KNOT_ERRNO_ERROR }
 	};
 	#undef ERR_ITEM
 
@@ -190,17 +239,41 @@ inline static int knot_map_errno_code(int code)
 		err++;
 	}
 
-	return err->libknot_code;
+	return err->errno_code != 0 ? err->libknot_code : dflt_error;
+}
+
+/*!
+ * \brief Map POSIX errno code to Knot error code.
+ *
+ * \param code Errno code to transform (set -1 to use the current errno).
+ *
+ * \return Mapped errno or KNOT_ERRNO_ERROR if unknown.
+ */
+inline static int knot_map_errno_code(int code)
+{
+	return knot_map_errno_code_def(code, KNOT_ERRNO_ERROR);
 }
 
 /*!
  * \brief Get a POSIX errno mapped to Knot error code.
  *
- * \return Mapped errno or KNOT_ERROR if unknown.
+ * \param dflt_error Default return value, if code is unknown.
+ *
+ * \return Mapped errno.
+ */
+inline static int knot_map_errno_def(int dflt_error)
+{
+	return knot_map_errno_code_def(-1, dflt_error);
+}
+
+/*!
+ * \brief Get a POSIX errno mapped to Knot error code.
+ *
+ * \return Mapped errno or KNOT_ERRNO_ERROR if unknown.
  */
 inline static int knot_map_errno(void)
 {
-	return knot_map_errno_code(-1);
+	return knot_map_errno_code_def(-1, KNOT_ERRNO_ERROR);
 }
 
 /*! @} */
